from setuptools import setup

setup(
    name='pylrc',
    packages=['pylrc'],
    version='0.0.6',
    description='A library for parsing .lrc files',
    long_description=open('README.rst').read(),
    author='doakey3',
    author_email='pylrc.dmodo@spamgourmet.com',
    url='https://github.com/doakey3/pylrc',
    download_url='https://github.com/doakey3/pylrc/releases/download/0.0.6/pylrc.tar.gz',
    license='MIT',
)
